@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: main.c,v 1.28.2.3 2004/05/24 10:49:23 fuyu Exp $
+ * $Id: main.c,v 1.28.2.5 2004/06/08 18:01:59 fuyu Exp $
  */
 
 #include "config.h"
@@ -74,6 +74,9 @@ ochusha_gdk_threads_enter(void)
   if (lock_depth > 1)
     fprintf(stderr, "gdk_threads_enter(): depth=%d\n", lock_depth);
 #endif
+#if ENABLE_STRICT_CHECK		/* Ѥθå */
+  g_assert(lock_depth == 1);
+#endif
 }
 
 
@@ -245,7 +248,7 @@ main(int argc, char *argv[])
   gtk_rc_add_default_file(PKGDATADIR "/" OCHUSHA_GTKRC);
 
   g_thread_init(NULL);
-#if GTK_MINOR_VERSION > 2
+#if 1 || GTK_MINOR_VERSION > 2
   gdk_threads_set_lock_functions(ochusha_gdk_threads_enter,
 				 ochusha_gdk_threads_leave);
 #endif
