@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: response_editor.h,v 1.8.2.1 2004/05/03 06:58:34 fuyu Exp $
+ * $Id: response_editor.h,v 1.8.2.2 2004/06/11 12:02:34 fuyu Exp $
  */
 
 #ifndef _RESPONSE_EDITOR_H_
@@ -80,7 +80,7 @@ struct _ResponseEditorClass
 };
 
 
-GType response_editor_get_type(void);
+GType response_editor_get_type(void) G_GNUC_CONST;
 
 GtkWidget *response_editor_new(void);
 
