@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_async_buffer.c,v 1.34 2004/03/21 20:43:48 fuyu Exp $
+ * $Id: ochusha_async_buffer.c,v 1.34.2.3 2004/06/06 15:39:14 fuyu Exp $
  */
 
 #include "config.h"
@@ -33,6 +33,7 @@
 #include "ochusha_async_buffer.h"
 #include "marshal.h"
 
+#include "monitor.h"
 #include "worker.h"
 
 #include <glib.h>
@@ -100,31 +101,17 @@ ochusha_async_buffer_get_type(void)
 }
 
 
-struct _BufferSyncObject
-{
-  pthread_mutex_t mutex;
-  pthread_cond_t cond;
-};
-
-
-static BufferSyncObject *get_sync_object(void);
+static Monitor *get_buffer_monitor(void);
 
 static GObjectClass *parent_class = NULL;
-static pthread_mutex_t ochusha_async_buffer_global_lock;
-static pthread_cond_t ochusha_async_buffer_global_condition;
+static Monitor *global_monitor = NULL;
 
 
 /*
  * pthreadϢΥ֥ȤλȤΤƤƧߤʤΤǡѤ
  * ݤ롣
  */
-static GSList *sync_object_list = NULL;
-
-/*
- * 饤֤(newƤfinalizeޤǤδ֤)Ƥ
- * OchushaAsyncBufferݻ
- */
-static GSList *ochusha_async_buffer_list = NULL;
+static GSList *free_monitor_list = NULL;
 
 /*
  * Active(number_of_active_users > 0)Ƥ
@@ -133,114 +120,32 @@ static GSList *ochusha_async_buffer_list = NULL;
 static GSList *active_buffer_list = NULL;
 
 
-#define LOCK_GLOBAL_LOCK						\
-  if (pthread_mutex_lock(&ochusha_async_buffer_global_lock) != 0)	\
-    {									\
-      fprintf(stderr, "Couldn't lock a mutex.\n");			\
-      abort();								\
-    }
-
-#define UNLOCK_GLOBAL_LOCK						\
-  if (pthread_mutex_unlock(&ochusha_async_buffer_global_lock) != 0)	\
-    {									\
-      fprintf(stderr, "Couldn't unlock a mutex.\n");			\
-      abort();								\
-    }
-
-#define WAIT_GLOBAL_CONDITION						\
-  if (pthread_cond_wait(&ochusha_async_buffer_global_condition,		\
-			&ochusha_async_buffer_global_lock) != 0)	\
-    {									\
-      fprintf(stderr, "Couldn't wait a condition.\n");			\
-      abort();								\
-    }
-
-#define SIGNAL_GLOBAL_CONDITION						\
-  if (pthread_cond_signal(&ochusha_async_buffer_global_condition) != 0)	\
-    {									\
-      fprintf(stderr, "Couldn't signal a condition.\n");		\
-      abort();								\
-    }
-
 #define LOCK_BUFFER_LOCK(buffer)					\
   do									\
     {									\
-      int result;							\
-      if (buffer->sync_object == NULL)					\
-	break;								\
-      result = pthread_mutex_trylock(&buffer->sync_object->mutex);	\
-      if (result != 0)							\
+      if (!ochusha_monitor_try_enter(buffer->monitor))			\
 	{								\
-	  if (result == EBUSY)						\
-	    {								\
-	      LOCK_GLOBAL_LOCK;						\
-	      buffer->number_of_lock_waiters++;				\
-	      UNLOCK_GLOBAL_LOCK;					\
-	      if (pthread_mutex_lock(&buffer->sync_object->mutex) != 0)	\
-		{							\
-		  fprintf(stderr, "Couldn't lock a mutex.\n");		\
-		  abort();						\
-		}							\
-	      LOCK_GLOBAL_LOCK;						\
-	      buffer->number_of_lock_waiters--;				\
-	      UNLOCK_GLOBAL_LOCK;					\
-	    }								\
-	  else								\
-	    {								\
-	      fprintf(stderr, "Couldn't trylock a mutex.\n");		\
-	      abort();							\
-	    }								\
+	  ochusha_monitor_enter(global_monitor);			\
+	  buffer->number_of_lock_waiters++;				\
+	  ochusha_monitor_exit(global_monitor);				\
+	  ochusha_monitor_enter(buffer->monitor);			\
+	  ochusha_monitor_enter(global_monitor);			\
+	  buffer->number_of_lock_waiters--;				\
+	  ochusha_monitor_exit(global_monitor);				\
 	}								\
     } while (0)
 
-#define UNLOCK_BUFFER_LOCK(buffer)					\
-  do									\
-    {									\
-      if (buffer->sync_object == NULL)					\
-	break;								\
-      if (pthread_mutex_unlock(&buffer->sync_object->mutex) != 0)	\
-	{								\
-	  fprintf(stderr, "Couldn't unlock a mutex.\n");		\
-	  abort();							\
-	}								\
-    } while (0)
+#define UNLOCK_BUFFER_LOCK(buffer)				\
+  do { ochusha_monitor_exit(buffer->monitor); } while (0)
 
 #define WAIT_BUFFER_CONDITION(buffer)				\
-  do								\
-    {								\
-      if (buffer->sync_object == NULL)				\
-	break;							\
-      if (pthread_cond_wait(&buffer->sync_object->cond,		\
-			    &buffer->sync_object->mutex) != 0)	\
-	{							\
-	  fprintf(stderr, "Couldn't wait a condition.\n");	\
-	  abort();						\
-	}							\
-    } while (0)
+  do { ochusha_monitor_wait(buffer->monitor); } while (0)
 
 #define SIGNAL_BUFFER_CONDITION(buffer)				\
-  do								\
-    {								\
-      if (buffer->sync_object == NULL)				\
-	break;							\
-      if (pthread_cond_signal(&buffer->sync_object->cond) != 0)	\
-	{							\
-	  fprintf(stderr, "Couldn't siganl a condition.\n");	\
-	  abort();						\
-	}							\
-    } while (0)
+  do { ochusha_monitor_notify(buffer->monitor); } while (0)
 
-#define BROADCAST_BUFFER_CONDITION(buffer)				\
-  do									\
-    {									\
-      if (buffer->sync_object == NULL)					\
-	break;								\
-      if (pthread_cond_broadcast(&buffer->sync_object->cond) != 0)	\
-	{								\
-	  fprintf(stderr, "Couldn't siganl a condition.\n");		\
-	  abort();							\
-	}								\
-    } while (0)
+#define BROADCAST_BUFFER_CONDITION(buffer)			\
+  do { ochusha_monitor_notify_all(buffer->monitor); } while (0)
 
 
 enum {
@@ -266,9 +171,6 @@ blind_awake(OchushaAsyncBuffer *buffer)
   if (buffer->state == OCHUSHA_ASYNC_BUFFER_OK
       && buffer->number_of_active_users > 0)
     {
-#if 0
-      fprintf(stderr, "blind awake on %p\n", buffer);
-#endif
       BROADCAST_BUFFER_CONDITION(buffer);
     }
 
@@ -282,11 +184,10 @@ static GSList *to_be_signaled_buffer_list = NULL;
 
 
 static void
-iterate_buffers_to_be_signaled(OchushaAsyncBuffer *buffer)
+iterate_buffers(OchushaAsyncBuffer *buffer, GSList **list)
 {
   g_object_ref(G_OBJECT(buffer));	/* ڥˤʤäƤϤ */
-  to_be_signaled_buffer_list = g_slist_append(to_be_signaled_buffer_list,
-					      buffer);
+  *list = g_slist_append(*list, buffer);
 }
 
 
@@ -299,11 +200,11 @@ force_awake_job(WorkerThread *employee, void *unused)
 
   while (TRUE)
     {
-      LOCK_GLOBAL_LOCK;
+      ochusha_monitor_enter(global_monitor);
       to_be_signaled_buffer_list = NULL;
       g_slist_foreach(active_buffer_list,
-		      (GFunc)iterate_buffers_to_be_signaled, NULL);
-      UNLOCK_GLOBAL_LOCK;
+		      (GFunc)iterate_buffers, &to_be_signaled_buffer_list);
+      ochusha_monitor_exit(global_monitor);
 
       g_slist_foreach(to_be_signaled_buffer_list, (GFunc)blind_awake, NULL);
 
@@ -324,17 +225,7 @@ ochusha_async_buffer_class_init(OchushaAsyncBufferClass *klass)
   GObjectClass *o_class = (GObjectClass *)klass;
   WorkerJob *job = G_NEW0(WorkerJob, 1);
 
-  if (pthread_mutex_init(&ochusha_async_buffer_global_lock, NULL) != 0)
-    {
-      fprintf(stderr, "Couldn't initialize a mutex.\n");
-      abort();
-    }
-
-  if (pthread_cond_init(&ochusha_async_buffer_global_condition, NULL) != 0)
-    {
-      fprintf(stderr, "Couldn't initialize a condvar.\n");
-      abort();
-    }
+  global_monitor = ochusha_monitor_new(NULL);
 
   parent_class = g_type_class_peek_parent(klass);
 
@@ -406,6 +297,14 @@ ochusha_async_buffer_init(OchushaAsyncBuffer *buffer)
 {
   buffer->fixed = FALSE;
   buffer->number_of_lock_waiters = 0;
+
+  ochusha_monitor_enter(global_monitor);
+  buffer->monitor = get_buffer_monitor();
+  ochusha_monitor_exit(global_monitor);
+#if DEBUG_MEMORY_USAGE
+  fprintf(stderr, "ochusha_async_buffer_init: OchushaAsyncBuffer(%p)\n",
+	  buffer);
+#endif
 }
 
 
@@ -413,79 +312,61 @@ static void
 ochusha_async_buffer_finalize(GObject *object)
 {
   OchushaAsyncBuffer *buffer = OCHUSHA_ASYNC_BUFFER(object);
-#if DEBUG_OCHUSHA_MEMORY_USAGE
+#if DEBUG_MEMORY_USAGE
   fprintf(stderr, "ochusha_async_buffer_finalize: OchushaAsyncBuffer(%p)\n",
 	  object);
 #endif
   if (buffer->destructor != NULL)
-    (*buffer->destructor)(buffer);
-
-  LOCK_GLOBAL_LOCK;
-  if (buffer->sync_object != NULL)
     {
-      sync_object_list = g_slist_append(sync_object_list, buffer->sync_object);
-      buffer->sync_object = NULL;
+      (*buffer->destructor)(buffer);
+      buffer->destructor = NULL;
+    }
+
+  ochusha_monitor_enter(global_monitor);
+  if (buffer->monitor != NULL)
+    {
+      free_monitor_list = g_slist_append(free_monitor_list, buffer->monitor);
+      buffer->monitor = NULL;
     }
-  ochusha_async_buffer_list = g_slist_remove(ochusha_async_buffer_list,
-					     buffer);
   if (g_slist_find(active_buffer_list, buffer) != NULL)
     {
       active_buffer_list = g_slist_remove(active_buffer_list, buffer);
       g_warning("OchushaAsyncBuffer(%p) is finalized in active state!.\n",
 		buffer);
     }
-  UNLOCK_GLOBAL_LOCK;
+  ochusha_monitor_exit(global_monitor);
 
   if (G_OBJECT_CLASS(parent_class)->finalize)
     (*G_OBJECT_CLASS(parent_class)->finalize)(object);
 }
 
 
-static BufferSyncObject *
-get_sync_object(void)
+static Monitor *
+get_buffer_monitor(void)
 {
-  GSList *recycled_sync_object;
-  BufferSyncObject *sync_object;
-
-  recycled_sync_object = sync_object_list;
-  sync_object_list = g_slist_remove_link(sync_object_list,
-					 recycled_sync_object);
-  if (recycled_sync_object != NULL)
+  /* global_monitorƱ */
+  Monitor *monitor;
+  GSList *recycled_monitor = free_monitor_list;
+  if (recycled_monitor != NULL)
     {
-      sync_object = (BufferSyncObject *)recycled_sync_object->data;
-      g_slist_free_1(recycled_sync_object);
-      return sync_object;
+      free_monitor_list
+	= g_slist_remove_link(free_monitor_list, recycled_monitor);
+      monitor = (Monitor *)recycled_monitor->data;
+      g_slist_free_1(recycled_monitor);
+      return monitor;
     }
 
-  /* Υ֥ȤϽλޤǻȤ󤹡*/
-  sync_object = (BufferSyncObject *)g_malloc(sizeof(BufferSyncObject));
-  if (pthread_mutex_init(&sync_object->mutex, NULL) != 0)
-    {
-      fprintf(stderr, "Couldn't init a mutex.\n");
-      abort();
-    }
-  if (pthread_cond_init(&sync_object->cond, NULL) != 0)
-    {
-      fprintf(stderr, "Couldn't init a condition variable.\n");
-      abort();
-    }
-  return sync_object;
+  /* MonitorϽλޤǻȤ󤵤롣*/
+  return ochusha_monitor_new(global_monitor);
 }
 
 
 OchushaAsyncBuffer *
 ochusha_async_buffer_new(char *buffer, size_t length, DestructFunc *destructor)
 {
-  BufferSyncObject *sync_object;
   OchushaAsyncBuffer *buf
     = OCHUSHA_ASYNC_BUFFER(g_object_new(OCHUSHA_TYPE_ASYNC_BUFFER, NULL));
 
-  LOCK_GLOBAL_LOCK;
-  sync_object = get_sync_object();
-  buf->sync_object = sync_object;
-  ochusha_async_buffer_list = g_slist_append(ochusha_async_buffer_list, buf);
-  UNLOCK_GLOBAL_LOCK;
-
   if (buffer == NULL && destructor == NULL)
     {
       if (length != 0)
@@ -514,18 +395,11 @@ ochusha_async_buffer_new(char *buffer, size_t length, DestructFunc *destructor)
 OchushaAsyncBuffer *
 ochusha_async_buffer_new_with_file(int fd)
 {
-  BufferSyncObject *sync_object;
   OchushaAsyncBuffer *buf
     = OCHUSHA_ASYNC_BUFFER(g_object_new(OCHUSHA_TYPE_ASYNC_BUFFER, NULL));
   gzFile gzfile;
   int chunk_size;
 
-  LOCK_GLOBAL_LOCK;
-  sync_object = get_sync_object();
-  buf->sync_object = sync_object;
-  ochusha_async_buffer_list = g_slist_append(ochusha_async_buffer_list, buf);
-  UNLOCK_GLOBAL_LOCK;
-
   buf->fixed = FALSE;
 
   buf->destructor = ochusha_async_buffer_free_when_finished;
@@ -556,15 +430,6 @@ ochusha_async_buffer_new_with_file(int fd)
 	  break;
 	}
 
-#if 0
-      fprintf(stderr, "0x%x bytes read\n", result);
-
-      if (result == 0)
-	fprintf(stderr, "EOF\n");
-      else if (result < chunk_size)
-	fprintf(stderr, "Close to EOF\n");
-#endif
-
       buf->length += result;
     }
 
@@ -592,18 +457,11 @@ munmap_when_finished(OchushaAsyncBuffer *buffer)
 OchushaAsyncBuffer *
 ochusha_async_buffer_new_with_file_mmap(int fd)
 {
-  BufferSyncObject *sync_object;
   char *buffer;
   OchushaAsyncBuffer *buf
     = OCHUSHA_ASYNC_BUFFER(g_object_new(OCHUSHA_TYPE_ASYNC_BUFFER, NULL));
   off_t len;
 
-  LOCK_GLOBAL_LOCK;
-  sync_object = get_sync_object();
-  buf->sync_object = sync_object;
-  ochusha_async_buffer_list = g_slist_append(ochusha_async_buffer_list, buf);
-  UNLOCK_GLOBAL_LOCK;
-
   len = lseek(fd, 0, SEEK_END);
   buffer = mmap(NULL, len, PROT_READ, MAP_NOCORE | MAP_PRIVATE, fd, 0);
   if (buffer == MAP_FAILED)
@@ -688,9 +546,9 @@ ochusha_async_buffer_active_ref(OchushaAsyncBuffer *buffer)
     {
       buffer->number_of_suspended_users++;
 
-      LOCK_GLOBAL_LOCK;
-      SIGNAL_GLOBAL_CONDITION;
-      UNLOCK_GLOBAL_LOCK;
+      ochusha_monitor_enter(global_monitor);
+      ochusha_monitor_notify(global_monitor);
+      ochusha_monitor_exit(global_monitor);
 
       WAIT_BUFFER_CONDITION(buffer);
     }
@@ -700,19 +558,19 @@ ochusha_async_buffer_active_ref(OchushaAsyncBuffer *buffer)
     {
       buffer->number_of_active_users--;
 
-      LOCK_GLOBAL_LOCK;
-      SIGNAL_GLOBAL_CONDITION;
-      UNLOCK_GLOBAL_LOCK;
+      ochusha_monitor_enter(global_monitor);
+      ochusha_monitor_notify(global_monitor);
+      ochusha_monitor_exit(global_monitor);
 
       g_object_unref(G_OBJECT(buffer));	/* Ĵ٤Ƥʤ */
     }
 
   if (buffer->number_of_active_users == 1)
     {
-      LOCK_GLOBAL_LOCK;
+      ochusha_monitor_enter(global_monitor);
       g_assert(g_slist_find(active_buffer_list, buffer) == NULL);
       active_buffer_list = g_slist_append(active_buffer_list, buffer);
-      UNLOCK_GLOBAL_LOCK;
+      ochusha_monitor_exit(global_monitor);
     }
 
   UNLOCK_BUFFER_LOCK(buffer);
@@ -736,17 +594,17 @@ ochusha_async_buffer_active_unref(OchushaAsyncBuffer *buffer)
 
   if (buffer->state != OCHUSHA_ASYNC_BUFFER_OK)
     {
-      LOCK_GLOBAL_LOCK;
-      SIGNAL_GLOBAL_CONDITION;
-      UNLOCK_GLOBAL_LOCK;
+      ochusha_monitor_enter(global_monitor);
+      ochusha_monitor_notify(global_monitor);
+      ochusha_monitor_exit(global_monitor);
     }
 
   if (buffer->number_of_active_users == 0)
     {
-      LOCK_GLOBAL_LOCK;
+      ochusha_monitor_enter(global_monitor);
       g_assert(g_slist_find(active_buffer_list, buffer) != NULL);
       active_buffer_list = g_slist_remove(active_buffer_list, buffer);
-      UNLOCK_GLOBAL_LOCK;
+      ochusha_monitor_exit(global_monitor);
     }
 
   UNLOCK_BUFFER_LOCK(buffer);
@@ -755,40 +613,57 @@ ochusha_async_buffer_active_unref(OchushaAsyncBuffer *buffer)
 }
 
 
-/*
- * BUG: Ǥactive_buffer_list˿äƤΤˡ
- *      GLOBAL_LOCKȤäƤʤ
- *      ƤӽФƤؿGLOBAL_LOCKȤäƤΤǡñ
- *      LOCK/UNLOCKǰϤ櫓ˤ⤤ġġ
- *      GLOBAL_LOCKGLibrecursive mutexѹ٤
- *      Ȱֻפärecursive mutexcondvarϻȤʤ
- *      Τ(⤽semanticstrivialˤϷʤʤΤ)
- *      줫鸫ľɬפäݤ褦嵐
- *      Ƥ桼active_buffer_listmutexcondvarǻȤ
- *      mutexƱǤɬפ¿ʬʤΤǡǧƤ顢
- *      ̤mutexäƲ褷褦
- */
 void
 ochusha_async_buffer_suspend_all(void)
 {
+  GSList *to_be_suspended_list = NULL;
+  if (global_monitor == NULL)
+    return;
+  ochusha_monitor_enter(global_monitor);
   g_slist_foreach(active_buffer_list,
+		  (GFunc)iterate_buffers, &to_be_suspended_list);
+  ochusha_monitor_exit(global_monitor);
+
+  g_slist_foreach(to_be_suspended_list,
 		  (GFunc)ochusha_async_buffer_suspend, NULL);
+  g_slist_foreach(to_be_suspended_list, (GFunc)g_object_unref, NULL);
+  g_slist_free(to_be_suspended_list);
 }
 
 
 void
 ochusha_async_buffer_resume_all(void)
 {
+  GSList *to_be_resumed_list = NULL;
+  if (global_monitor == NULL)
+    return;
+  ochusha_monitor_enter(global_monitor);
   g_slist_foreach(active_buffer_list,
+		  (GFunc)iterate_buffers, &to_be_resumed_list);
+  ochusha_monitor_exit(global_monitor);
+
+  g_slist_foreach(to_be_resumed_list,
 		  (GFunc)ochusha_async_buffer_resume, NULL);
+  g_slist_foreach(to_be_resumed_list, (GFunc)g_object_unref, NULL);
+  g_slist_free(to_be_resumed_list);
 }
 
 
 void
 ochusha_async_buffer_terminate_all(void)
 {
+  GSList *to_be_terminated_list = NULL;
+  if (global_monitor == NULL)
+    return;
+  ochusha_monitor_enter(global_monitor);
   g_slist_foreach(active_buffer_list,
+		  (GFunc)iterate_buffers, &to_be_terminated_list);
+  ochusha_monitor_exit(global_monitor);
+
+  g_slist_foreach(to_be_terminated_list,
 		  (GFunc)ochusha_async_buffer_terminate, NULL);
+  g_slist_foreach(to_be_terminated_list, (GFunc)g_object_unref, NULL);
+  g_slist_free(to_be_terminated_list);
 }
 
 
@@ -796,14 +671,10 @@ void
 ochusha_async_buffer_suspend(OchushaAsyncBuffer *buffer)
 {
   gboolean dead_buffer;
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr, "suspend threads refering OchushaAsyncBuffer(%p)\n",
-	  buffer);
-#endif
 
-  LOCK_GLOBAL_LOCK;
+  ochusha_monitor_enter(global_monitor);
   dead_buffer = g_slist_find(active_buffer_list, buffer) == NULL;
-  UNLOCK_GLOBAL_LOCK;
+  ochusha_monitor_exit(global_monitor);
   if (dead_buffer)
     return;
 
@@ -811,43 +682,28 @@ ochusha_async_buffer_suspend(OchushaAsyncBuffer *buffer)
   buffer->number_of_suspended_users = 0;
   buffer->state = OCHUSHA_ASYNC_BUFFER_SUSPENDED;
   BROADCAST_BUFFER_CONDITION(buffer);
-  LOCK_GLOBAL_LOCK;
+
+  ochusha_monitor_enter(global_monitor);
 
   g_signal_emit(G_OBJECT(buffer),
 		async_buffer_signals[WAKEUP_NOW_SIGNAL],
 		0);
 
-  UNLOCK_BUFFER_LOCK(buffer);		/* νǤʤȤʤʤ
-					 * UNLOCK_BUFFER_LOCKִ֤
-					 * ΥåԤäƤåɤ
-					 * ưϤ롣åɤξ
-					 * åSUSPENDED֤Ǥ
-					 * ȤΤΤǡSUSPENDED֤
-					 * ֲ򤱤ΤԤġwait롣
-					 * BROADCASTϿSUSPENDED֤
-					 * ʤäȤΤʤåɤ
-					 * 뤿ΤΤǤ
-					 * ա
-					 */
-
   while (buffer->number_of_suspended_users < buffer->number_of_active_users)
     {
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-      fprintf(stderr,
-	      "ochusha_async_buffer_suspend: suspended=%d, active_users=%d\n",
-	      buffer->number_of_suspended_users,
-	      buffer->number_of_active_users);
+      UNLOCK_BUFFER_LOCK(buffer);
+      if (!ochusha_monitor_timedwait(global_monitor, 500))
+	{
+#if 0
+	  fprintf(stderr, "Some threads not responding...\n");
 #endif
-      WAIT_GLOBAL_CONDITION;
+	  goto finish;
+	}
+      LOCK_BUFFER_LOCK(buffer);
     }
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr,
-	  "ochusha_async_buffer_suspend: suspended=%d, active_users=%d\n",
-	  buffer->number_of_suspended_users,
-	  buffer->number_of_active_users);
-#endif
-
-  UNLOCK_GLOBAL_LOCK;
+  UNLOCK_BUFFER_LOCK(buffer);
+ finish:
+  ochusha_monitor_exit(global_monitor);
 }
 
 
@@ -856,9 +712,9 @@ ochusha_async_buffer_resume(OchushaAsyncBuffer *buffer)
 {
   gboolean dead_buffer;
 
-  LOCK_GLOBAL_LOCK;
+  ochusha_monitor_enter(global_monitor);
   dead_buffer = g_slist_find(active_buffer_list, buffer) == NULL;
-  UNLOCK_GLOBAL_LOCK;
+  ochusha_monitor_exit(global_monitor);
   if (dead_buffer)
     return;
 
@@ -873,14 +729,10 @@ void
 ochusha_async_buffer_terminate(OchushaAsyncBuffer *buffer)
 {
   gboolean dead_buffer;
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr,
-	  "ochusha_async_buffer_terminate: buffer=%p\n", buffer);
-#endif
 
-  LOCK_GLOBAL_LOCK;
+  ochusha_monitor_enter(global_monitor);
   dead_buffer = g_slist_find(active_buffer_list, buffer) == NULL;
-  UNLOCK_GLOBAL_LOCK;
+  ochusha_monitor_exit(global_monitor);
   if (dead_buffer)
     return;
 
@@ -888,46 +740,28 @@ ochusha_async_buffer_terminate(OchushaAsyncBuffer *buffer)
   buffer->state = OCHUSHA_ASYNC_BUFFER_TERMINATED;
   BROADCAST_BUFFER_CONDITION(buffer);
 
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr,
-	  "ochusha_async_buffer_terminate: buffer condition broadcast done\n");
-#endif
-
-  LOCK_GLOBAL_LOCK;
-
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr,
-	  "ochusha_async_buffer_terminate: global lock locked\n");
-#endif
+  ochusha_monitor_enter(global_monitor);
 
   g_signal_emit(G_OBJECT(buffer),
 		async_buffer_signals[WAKEUP_NOW_SIGNAL],
 		0);
 
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr,
-	  "ochusha_async_buffer_terminate: buffer lock unlocked\n");
-#endif
-
   while (buffer->number_of_active_users > 0)
     {
       UNLOCK_BUFFER_LOCK(buffer);
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-      fprintf(stderr,
-	      "ochusha_async_buffer_terminate: active_users=%d\n",
-	      buffer->number_of_active_users);
+      if (!ochusha_monitor_timedwait(global_monitor, 500))
+	{
+#if 0
+	  fprintf(stderr, "Some threads not responding...\n");
 #endif
-      WAIT_GLOBAL_CONDITION;
+	  goto finish;
+	}
       LOCK_BUFFER_LOCK(buffer);
     }
   UNLOCK_BUFFER_LOCK(buffer);
 
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr, "ochusha_async_buffer_terminate: active_users=%d\n",
-	  buffer->number_of_active_users);
-#endif
-
-  UNLOCK_GLOBAL_LOCK;
+ finish:
+  ochusha_monitor_exit(global_monitor);
 }
 
 
@@ -937,9 +771,8 @@ ochusha_async_buffer_terminate(OchushaAsyncBuffer *buffer)
 gboolean
 ochusha_async_buffer_update_length(OchushaAsyncBuffer *buffer, size_t length)
 {
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr, "ochusha_async_buffer_update_length called\n");
-#endif
+  if (buffer->state == OCHUSHA_ASYNC_BUFFER_TERMINATED)
+    return FALSE;	/* XXX: 䤷 */
   LOCK_BUFFER_LOCK(buffer);
   buffer->length = length;
   UNLOCK_BUFFER_LOCK(buffer);
@@ -954,10 +787,8 @@ ochusha_async_buffer_update_length(OchushaAsyncBuffer *buffer, size_t length)
 gboolean
 ochusha_async_buffer_fix(OchushaAsyncBuffer *buffer)
 {
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr, "ochusha_async_buffer_fix called\n");
-#endif
-
+  if (buffer->state == OCHUSHA_ASYNC_BUFFER_TERMINATED)
+    return FALSE;	/* XXX: 䤷 */
   LOCK_BUFFER_LOCK(buffer);
   buffer->fixed = TRUE;
   UNLOCK_BUFFER_LOCK(buffer);
@@ -972,10 +803,8 @@ ochusha_async_buffer_fix(OchushaAsyncBuffer *buffer)
 gboolean
 ochusha_async_buffer_resize(OchushaAsyncBuffer *buffer, size_t length)
 {
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr, "ochusha_async_buffer_resize called\n");
-#endif
-
+  if (buffer->state == OCHUSHA_ASYNC_BUFFER_TERMINATED)
+    return FALSE;	/* XXX: 䤷 */
   LOCK_BUFFER_LOCK(buffer);
 
   if (buffer->fixed)
@@ -1006,12 +835,6 @@ ochusha_async_buffer_ensure_free_space(OchushaAsyncBuffer *buffer,
 {
   gboolean result = TRUE;
 
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr,
-	  "ochusha_async_buffer_ensure_free_space(%p, %d) called.\n",
-	  buffer, length);
-#endif
-
   LOCK_BUFFER_LOCK(buffer);
   if (buffer->fixed)
     {
@@ -1022,16 +845,11 @@ ochusha_async_buffer_ensure_free_space(OchushaAsyncBuffer *buffer,
 
   while (buffer->state == OCHUSHA_ASYNC_BUFFER_SUSPENDED)
     {
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-      fprintf(stderr,
-	      "buffer(%p) is suspended at ochusha_async_buffer_free_space.\n",
-	      buffer);
-#endif
       buffer->number_of_suspended_users++;
 
-      LOCK_GLOBAL_LOCK;
-      SIGNAL_GLOBAL_CONDITION;
-      UNLOCK_GLOBAL_LOCK;
+      ochusha_monitor_enter(global_monitor);
+      ochusha_monitor_notify(global_monitor);
+      ochusha_monitor_exit(global_monitor);
 
       WAIT_BUFFER_CONDITION(buffer);
       /* λbuffer->state != OCHUSHA_ASYNC_BUFFER_SUSPENDEDʤ
@@ -1063,10 +881,6 @@ ochusha_async_buffer_append_data(OchushaAsyncBuffer *buffer, const char *data,
 {
   gboolean result = TRUE;
 
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr, "ochusha_async_buffer_append_data called.\n");
-#endif
-
   if (ochusha_async_buffer_ensure_free_space(buffer, length))
     {
       memcpy((char *)buffer->buffer + buffer->length, data, length);
@@ -1148,33 +962,19 @@ ochusha_async_buffer_check_active(OchushaAsyncBuffer *buffer)
 {
   gboolean result;
 
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr,
-	  "ochusha_async_buffer_check_active(%p)\n",
-	  buffer);
-#endif
-
   LOCK_BUFFER_LOCK(buffer);
 
   while (buffer->state == OCHUSHA_ASYNC_BUFFER_SUSPENDED)
     {
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-      fprintf(stderr,
-	      "buffer(%p) is suspended at ochusha_async_buffer_check_signal.\n",
-	      buffer);
-#endif
       buffer->number_of_suspended_users++;
 
-      LOCK_GLOBAL_LOCK;
-      SIGNAL_GLOBAL_CONDITION;
-      UNLOCK_GLOBAL_LOCK;
+      ochusha_monitor_enter(global_monitor);
+      ochusha_monitor_notify(global_monitor);
+      ochusha_monitor_exit(global_monitor);
 
       WAIT_BUFFER_CONDITION(buffer);	/* ԤäƤ륷ʥ
 					 * ᥤ󥹥åɤΤΡ
 					 */
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-      fprintf(stderr, "buffer(%p) resumed.\n", buffer);
-#endif
     }
   /* ХåեmutexunlockǤstateѤäƤǽΤ
    * λξ֤ǺοȤο롣Ĥޤꡢλ
@@ -1195,31 +995,19 @@ ochusha_async_buffer_signal(OchushaAsyncBuffer *buffer)
 {
   gboolean result;
 
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr, "ochusha_async_buffer_signal(%p)\n", buffer);
-#endif
-
   LOCK_BUFFER_LOCK(buffer);
 
   while (buffer->state == OCHUSHA_ASYNC_BUFFER_SUSPENDED)
     {
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-      fprintf(stderr,
-	      "buffer(%p) is suspended at ochusha_async_buffer_signal.\n",
-	      buffer);
-#endif
       buffer->number_of_suspended_users++;
 
-      LOCK_GLOBAL_LOCK;
-      SIGNAL_GLOBAL_CONDITION;
-      UNLOCK_GLOBAL_LOCK;
+      ochusha_monitor_enter(global_monitor);
+      ochusha_monitor_notify(global_monitor);
+      ochusha_monitor_exit(global_monitor);
 
       WAIT_BUFFER_CONDITION(buffer);	/* ԤäƤ륷ʥ
 					 * ᥤ󥹥åɤΤΡ
 					 */
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-      fprintf(stderr, "buffer(%p) resumed.\n", buffer);
-#endif
     }
   /* ХåեmutexunlockǤstateѤäƤǽΤ
    * λξ֤ǺοȤο롣Ĥޤꡢλ
@@ -1244,31 +1032,19 @@ ochusha_async_buffer_broadcast(OchushaAsyncBuffer *buffer)
 {
   gboolean result;
 
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr, "ochusha_async_buffer_broadcast(%p)\n", buffer);
-#endif
-
   LOCK_BUFFER_LOCK(buffer);
 
   while (buffer->state == OCHUSHA_ASYNC_BUFFER_SUSPENDED)
     {
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-      fprintf(stderr,
-	      "buffer(%p) is suspended at ochusha_async_buffer_broadcast.\n",
-	      buffer);
-#endif
       buffer->number_of_suspended_users++;
 
-      LOCK_GLOBAL_LOCK;
-      SIGNAL_GLOBAL_CONDITION;
-      UNLOCK_GLOBAL_LOCK;
+      ochusha_monitor_enter(global_monitor);
+      ochusha_monitor_notify(global_monitor);
+      ochusha_monitor_exit(global_monitor);
 
       WAIT_BUFFER_CONDITION(buffer);	/* ԤäƤ륷ʥ
 					 * ᥤ󥹥åɤΤΡ
 					 */
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-      fprintf(stderr, "buffer(%p) resumed.\n", buffer);
-#endif
     }
   /* ХåեmutexunlockǤstateѤäƤǽΤ
    * λξ֤ǺοȤο롣Ĥޤꡢλ
@@ -1289,30 +1065,18 @@ ochusha_async_buffer_broadcast(OchushaAsyncBuffer *buffer)
 gboolean
 ochusha_async_buffer_wait(OchushaAsyncBuffer *buffer)
 {
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr, "ochusha_async_buffer_wait(%p)\n", buffer);
-#endif
-
   while (buffer->state == OCHUSHA_ASYNC_BUFFER_SUSPENDED)
     {
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-      fprintf(stderr,
-	      "buffer(%p) is suspended at ochusha_async_buffer_wait.\n",
-	      buffer);
-#endif
       buffer->number_of_suspended_users++;
 
-      LOCK_GLOBAL_LOCK;
-      SIGNAL_GLOBAL_CONDITION;
-      UNLOCK_GLOBAL_LOCK;
+      ochusha_monitor_enter(global_monitor);
+      ochusha_monitor_notify(global_monitor);
+      ochusha_monitor_exit(global_monitor);
 
       WAIT_BUFFER_CONDITION(buffer);
       /* λbuffer->state != OCHUSHA_ASYNC_BUFFER_SUSPENDEDʤ
        * ʥsuspend򤯤˥ᥤ󥹥åɤäƤΡ
        */
-#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-      fprintf(stderr, "buffer(%p) resumed.\n", buffer);
-#endif
     }
 
   if (buffer->state != OCHUSHA_ASYNC_BUFFER_TERMINATED)
