@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_board_2ch.c,v 1.45 2004/04/13 15:38:05 fuyu Exp $
+ * $Id: ochusha_board_2ch.c,v 1.45.2.2 2004/06/04 18:57:55 fuyu Exp $
  */
 
 #include "config.h"
@@ -52,6 +52,7 @@
 #include <zlib.h>
 
 
+#define DEBUG_LIBOCHUSHA	0
 #define ANALYSIS_INTERVAL	200
 
 
@@ -242,6 +243,18 @@ add_a_server(OchushaBoard2ch *board_2ch, const char *server)
   g_return_if_fail(server != NULL);
   g_return_if_fail(strlen(server) > 0);
 
+  switch (ochusha_bulletin_board_get_bbs_type(OCHUSHA_BULLETIN_BOARD(board_2ch)))
+    {
+    case OCHUSHA_BBS_TYPE_2CH:
+      if (strstr(server, "2ch.net") == NULL)
+	return;
+      break;
+    case OCHUSHA_BBS_TYPE_MACHIBBS:
+      if (strstr(server, "machi.to") == NULL)
+	return;
+      break;
+    }
+
   if (g_slist_find_custom(board_2ch->server_history, server,
 			  (GCompareFunc)strcmp) == NULL)
     board_2ch->server_history = g_slist_prepend(board_2ch->server_history,
@@ -453,8 +466,8 @@ ochusha_board_2ch_recover_threadlist(OchushaBulletinBoard *board,
 						     NULL);
 		  OCHU_OBJECT_UNREF(dat_buffer);
 		  thread->number_of_responses_read = 1;
-		  board->thread_list = g_slist_append(board->thread_list,
-						      thread);
+		  board->thread_list = g_slist_prepend(board->thread_list,
+						       thread);
 		}
 	    }
 	}
@@ -524,10 +537,6 @@ ochusha_board_2ch_server_changed(OchushaBulletinBoard *board,
   g_return_if_fail(OCHUSHA_IS_BOARD_2CH(board) && prev_server != NULL);
 
   board_2ch = OCHUSHA_BOARD_2CH(board);
-  /* XXX server_historyϿ롣
-   *     boardlist.xmlؤν񤭹߻˥ꥹȤ񤭽Ф
-   *     xmlǤϤɤäʡġġ
-   */
   add_a_server(board_2ch, prev_server);
 }
 
@@ -638,8 +647,8 @@ ochusha_board_2ch_lookup_kako_thread_by_url(OchushaBulletinBoard *board,
 	      if (thread->title != NULL)
 		{
 		  thread->number_of_responses_read = 1;
-		  board->thread_list = g_slist_append(board->thread_list,
-						      thread);
+		  board->thread_list = g_slist_prepend(board->thread_list,
+						       thread);
 		}
 	      else
 		{
@@ -1003,7 +1012,7 @@ collect_dropped_thread(gpointer data, gpointer user_data)
 	  if (g_slist_find(args->thread_list, thread) == NULL)
 	    {
 	      thread->number_of_responses_on_server = 0;
-	      args->thread_list = g_slist_append(args->thread_list, thread);
+	      args->thread_list = g_slist_prepend(args->thread_list, thread);
 	      if (args->each_thread_cb != NULL)
 		if (!(*args->each_thread_cb)(thread, args->callback_data))
 		  args->each_thread_cb = FALSE;
@@ -1015,7 +1024,7 @@ collect_dropped_thread(gpointer data, gpointer user_data)
 	}
       else
 	thread->board->dropped_list
-	  = g_slist_append(thread->board->dropped_list, thread);
+	  = g_slist_prepend(thread->board->dropped_list, thread);
     }
   else
     OCHU_OBJECT_UNREF(data);	/* Ƥ륹thread_list
@@ -1141,7 +1150,10 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
 	    if (tmp_pos == NULL)
 	      {
 #if DEBUG_LIBOCHUSHA
-		fprintf(stderr, "Unknown format found in subject.txt.\n");
+		tmp_pos = g_strndup(cur_pos, eol_pos - cur_pos);
+		fprintf(stderr, "Unknown format found in subject.txt: %s\n",
+			tmp_pos);
+		g_free(tmp_pos);
 #endif
 		goto prepare_next_line;
 	      }
@@ -1184,6 +1196,10 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
 								    thread_id);
 	    if (thread == NULL)
 	      {
+#if DEBUG_LIBOCHUSHA
+		fprintf(stderr, "thread not found: %s\n", thread_id);
+#endif
+
 		thread_title = simple_string_canon(title_pos, title_len,
 						   converter,
 						   converter_helper);
@@ -1192,6 +1208,10 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
 							       thread_title);
 		G_FREE(thread_title);
 	      }
+#if DEBUG_LIBOCHUSHA
+	    else
+	      fprintf(stderr, "thread found: %s\n", thread_id);
+#endif
 
 	    if (thread_id != default_buffer)
 	      G_FREE(thread_id);
@@ -1206,7 +1226,7 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
 		thread->number_of_responses_on_server = n_responses_on_server;
 
 		OCHU_OBJECT_REF(thread);
-		thread_list = g_slist_append(thread_list, thread);
+		thread_list = g_slist_prepend(thread_list, thread);
 
 		if (each_thread_cb != NULL
 		    && !(*each_thread_cb)(thread, callback_data))
